<?php

class Ajax
{

    public function __construct()
    {
        self::init();
    }

    public static function init()
    {
        add_action('init', array(__class__, 'define_ajax'), 0);
        add_action('template_redirect', array(__class__, 'do_wc_ajax'), 0);
        self::add_ajax_events();
    }

    public static function get_endpoint($request = '')
    {
        return esc_url_raw(add_query_arg('wc-ajax', $request, remove_query_arg(array('remove_item', 'add-to-cart', 'added-to-cart'))));
    }

    public static function define_ajax()
    {
        if (!empty($_GET['wc-ajax'])) {
            if (!defined('DOING_AJAX')) {
                define('DOING_AJAX', true);
            }
            if (!defined('WC_DOING_AJAX')) {
                define('WC_DOING_AJAX', true);
            }
            // Turn off display_errors during AJAX events to prevent malformed JSON
            if (!WP_DEBUG || (WP_DEBUG && !WP_DEBUG_DISPLAY)) {
                @ini_set('display_errors', 0);
            }
            $GLOBALS['wpdb']->hide_errors();
        }
    }

    private static function wc_ajax_headers()
    {
        send_origin_headers();
        @header('Content-Type: text/html; charset=' . get_option('blog_charset'));
        @header('X-Robots-Tag: noindex');
        send_nosniff_header();
        nocache_headers();
        status_header(200);
    }

    public static function do_wc_ajax()
    {
        global $wp_query;
        if (!empty($_GET['wc-ajax'])) {
            $wp_query->set('wc-ajax', sanitize_text_field($_GET['wc-ajax']));
        }
        if ($action = $wp_query->get('wc-ajax')) {
            self::wc_ajax_headers();
            do_action('wc_ajax_' . sanitize_text_field($action));
            die();
        }
    }

    public static function add_ajax_events()
    {
        // woocommerce_EVENT => nopriv
        $ajax_events = array(
            'logistra_get_service_partners' => true,
            'logistra_update_apikey' => false,
            'logistra_update_token' => false,
            'logistra_update_settings_current_tab' => false,
            'logistra_update_general_settings' => false,
            'logistra_get_token' => false,
            'logistra_update_sender' => false,
            'logistra_get_shipping_methods_all' => false,
            'logistra_update_transport_products' => false,
            'logistra_update_shipping_method_relation' => false,
            'logistra_create_consignments_for_order' => false,
            'logistra_return_consignments_for_order' => false,
            'logistra_get_printers' => false,
            'logistra_update_wildrobot_username' => false,
            'logistra_update_print_settings' => false,
            'logistra_get_transport_products' => false,
            'logistra_update_return_address' => false,
        );
        foreach ($ajax_events as $ajax_event => $nopriv) {
            add_action('wp_ajax_woocommerce_' . $ajax_event, array(__class__, $ajax_event));
            if ($nopriv) {
                add_action('wp_ajax_nopriv_woocommerce_' . $ajax_event, array(__class__, $ajax_event));
                // WC AJAX can be used for frontend ajax requests
                add_action('wc_ajax_' . $ajax_event, array(__class__, $ajax_event));
            }
        }
    }

    /* BACKEND METHODS */

    // Settings
    public static function logistra_update_wildrobot_username()
    {
        self::check_security();
        $value = sanitize_text_field($_REQUEST['value']);
        if ($value === "null") {
            $value = null;
        };

        update_option('logistra-robots-wildrobot-username', $value, '', null);

        wp_send_json_success(array(
            'value' => $value,
            'message' => __('Instilling oppdatert.', 'logistra-robots')
        ));
    }

    public static function logistra_update_apikey()
    {
        self::check_security();
        $value = sanitize_text_field($_REQUEST['value']);

        // Since we are actually updateing the API key to something else we should clear some options. 
        // require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-logistra-robots-helpers.php';
        // Logistra_Robots_Helpers::clear_all_options();

        if (!update_option('logistra-robots-apikey', $value, '', false)) {
            wp_send_json_success(array(
                'value' => $value,
                'message' => "API nøkkel oppdatert."
            ));
        }

        wp_send_json_success(array(
            'value' => $value,
            'message' => "API nøkkel oppdatert.",
            "reRender" => true,
        ));
    }

    public static function logistra_update_settings_current_tab()
    {
        self::check_security();
        $value = sanitize_text_field($_REQUEST['value']);

        if (!update_option('logistra-robots-settings-current-tab', $value, '', false)) {
            wp_send_json_success(array(
                'value' => $value,
                'message' => "Nåværende fane oppdatert. Ingen endring."
            ));
        }

        wp_send_json_success(array(
            'value' => $value,
            'message' => "Nåværende fane oppdatert."
        ));
    }

    public static function logistra_update_print_settings()
    {
        self::check_security();
        $printerDefault = self::check_field('printerDefault');
        $printerInterval = self::check_field('printerInterval');
        $printerIntervalTime = self::check_field('printerIntervalTime');
        $transferMethod = self::check_field('transferMethod');
        $transferTime = self::check_field('transferTime');



        update_option('logistra-robots-printer-default', $printerDefault, '', false);
        update_option('logistra-robots-printer-interval', $printerInterval, '', false);
        update_option('logistra-robots-printer-interval-time', $printerIntervalTime, '', false);
        update_option('logistra-robots-selected-transfer-method', $transferMethod, '', false);
        update_option('logistra-robots-selected-transfer-time', $transferTime, '', false);


        wp_send_json_success(array(
            'message' => "Fraktinstillinger oppdatert."
        ));
    }

    public static function logistra_update_return_address()
    {
        self::check_security();
        $address1 = self::check_field('address1');
        $address2 = self::check_field('address2');
        $city = self::check_field('city');
        $country = self::check_field('country');
        $postcode = self::check_field('postcode');

        update_option('logistra-robots-return-address-address1', $address1, '', false);
        update_option('logistra-robots-return-address-address2', $address2, '', false);
        update_option('logistra-robots-return-address-city', $city, '', false);
        update_option('logistra-robots-return-address-country', $country, '', false);
        update_option('logistra-robots-return-address-postcode', $postcode, '', false);


        wp_send_json_success(array(
            'message' => "Return addresse oppdatert."
        ));
    }

    public static function logistra_update_general_settings()
    {
        self::check_security();
        $completeOrder = sanitize_text_field($_REQUEST['completeOrder']);
        $sendConsignmentOnCompleteOrder = sanitize_text_field($_REQUEST['sendConsignmentOnCompleteOrder']);
        $pickupPointCart = sanitize_text_field($_REQUEST['pickupPointCart']);
        $pickupPointCheckout = sanitize_text_field($_REQUEST['pickupPointCheckout']);
        $addOrgNameToOrder = sanitize_text_field($_REQUEST['addOrgNameToOrder']);
        $staticWeightOnOrders = sanitize_text_field($_REQUEST['staticWeightOnOrders']);
        $staticWeightInKg = sanitize_text_field($_REQUEST['staticWeightInKg']);
        $consignmentDescriptionProductNames = sanitize_text_field($_REQUEST['consignmentDescriptionProductNames']);
        $useTableRateIntegration = sanitize_text_field($_REQUEST['useTableRateIntegration']);
        $activatedManualPickup = sanitize_text_field($_REQUEST['activatedManualPickup']);
        $freightTrackUrlEmail = sanitize_text_field($_REQUEST['freightTrackUrlEmail']);
        $freeFreightNotice = sanitize_text_field($_REQUEST['freeFreightNotice']);
        $freeFreightAlmostValue = sanitize_text_field($_REQUEST['freeFreightAlmostValue']);

        if (!update_option('logistra-robots-setting-complete-order', $completeOrder === "true" ? 'yes' : 'no', '', false)) {
        }
        if (!update_option('logistra-robots-setting-send-consignment-on-complete-order', $sendConsignmentOnCompleteOrder === "true" ? 'yes' : 'no', '', false)) {
        }
        if (!update_option('logistra-robots-pickuppoint-cart', $pickupPointCart === "true" ? 'yes' : 'no', '', false)) {
        }
        if (!update_option('logistra-robots-pickuppoint-checkout', $pickupPointCheckout === "true" ? 'yes' : 'no', '', false)) {
        }
        if (!update_option('logistra-robots-add-org-name-to-order', $addOrgNameToOrder === "true" ? 'yes' : 'no', '', false)) {
        }
        if (!update_option('logistra-robots-static-weight-on-orders', $staticWeightOnOrders === "true" ? 'yes' : 'no', '', false)) {
        }
        if (!update_option('logistra-robots-static-weight-in-kg', $staticWeightInKg, '', false)) {
        }
        if (!update_option('logistra-robots-consignment-description-product-names', $consignmentDescriptionProductNames === "true" ? 'yes' : 'no', '', false)) {
        }
        if (!update_option('logistra-robots-use-table-rate-integration', $useTableRateIntegration === "true" ? 'yes' : 'no', '', false)) {
        }
        if (!update_option('logistra-robots-activated-manual-pickup', $activatedManualPickup === "true" ? 'yes' : 'no', '', false)) {
        }
        if (!update_option('logistra-robots-freight-track-url-email', $freightTrackUrlEmail === "true" ? 'yes' : 'no', '', false)) {
        }
        if (!update_option('logistra-robots-free-freight-notice', $freeFreightNotice === "true" ? 'yes' : 'no', '', false)) {
        }
        if (!update_option('logistra-robots-free-freight-almost-value', $freeFreightAlmostValue, '', false)) {
        }

        wp_send_json_success(array(
            'message' => "Generelle instillinger oppdatert."
        ));
    }

    public static function logistra_update_token()
    {
        self::check_security();
        $value = sanitize_text_field($_REQUEST['value']);

        if (!update_option('logistra-robots-token', $value, '', false)) {
            wp_send_json_success(array(
                'value' => $value,
                'message' => "Token oppdatert. Ingen endring."
            ));
        }

        wp_send_json_success(array(
            'value' => $value,
            'message' => "Token oppdatert."
        ));
    }

    public static function logistra_get_token()
    {
        self::check_security();

        wp_send_json_success(array(
            'token' => get_option('logistra-robots-token', null),
        ));
    }

    public static function logistra_update_sender()
    {
        self::check_security();
        $value = sanitize_text_field($_REQUEST['value']);
        $name = sanitize_text_field($_REQUEST['name']);
        $senders = sanitize_textarea_field($_REQUEST['senders']);
        $message = "";
        // Since we are actually updateing the API key to something else we should clear some options. 
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-logistra-robots-helpers.php';
        Logistra_Robots_Helpers::clear_all_options();

        if (!update_option('logistra-robots-senderid', $value, '', false)) {
            $message .= "Oppdatert logistra avsender. (ingen endring)";
        } else {
            $message .= "Oppdatert logistra avsender.";
        }
        if (!update_option('logistra-robots-sender-name', $name, '', false)) {
            // $message .= "Logistra webshop sender name has same value as before. ";
        }
        if (!update_option('logistra-robots-senders', $senders, '', false)) {
            // $message .= "Logistra webshop senders name has same value as before. ";
        }
        Logistra_Robots_Helpers::clear_all_shipping_method_relations();

        wp_send_json_success(array(
            'value' => $value,
            'message' => $message
        ));
    }


    public static function logistra_get_transport_products()
    {
        self::check_security();
        $tp = get_option('logistra-robots-transport-agreements', wp_json_encode(new stdClass()));

        wp_send_json_success(array(
            'transportProducts' => $tp,
        ));
    }

    public static function logistra_update_transport_products()
    {
        self::check_security();
        $transportProducts = sanitize_textarea_field($_REQUEST['transportProducts']);

        !update_option('logistra-robots-transport-agreements', $transportProducts, '', false);

        wp_send_json_success(array(
            'value' => "Updated transportProducts",
        ));
    }

    public static function logistra_update_transport_agreements_hash()
    {
        self::check_security();
        $value = sanitize_text_field($_REQUEST['value']);
        $message = "";

        if (!update_option('logistra-robots-transport-agreements-hash', $value, '', false)) {
            //$message .= "Hash av transportprodukter oppdatert. Ingen endring.";
        }

        wp_send_json_success(array(
            'value' => $value,
            'message' => $message,
        ));
    }

    public static function logistra_create_consignments_for_order()
    {
        self::check_security();
        $orderId = sanitize_text_field($_REQUEST['orderId']);
        $productId = sanitize_text_field($_REQUEST['productId']);
        $transportAgreementId = sanitize_text_field($_REQUEST['transportAgreementId']);
        $packages = sanitize_text_field($_REQUEST['packages']);
        $overrideWeight = sanitize_text_field($_REQUEST['overrideWeight']);
        $weight = sanitize_text_field($_REQUEST['weight']);
        $length = sanitize_text_field($_REQUEST['length']);
        $width = sanitize_text_field($_REQUEST['width']);
        $height = sanitize_text_field($_REQUEST['height']);

        $productId = $productId === "null" ? null : $productId;
        $transportAgreementId = $transportAgreementId === "null" ? null : $transportAgreementId;
        $packages = $packages === "null" ? null : intval($packages);
        $overrideWeight = $overrideWeight === "null" ? null : $overrideWeight;
        $weight = $weight === "null" ? null : intval($weight);
        $length = $length === "null" ? null : intval($length);
        $width = $width === "null" ? null : intval($width);
        $height = $height === "null" ? null : intval($height);

        $logger = new WC_Logger();
        $context = array('source' => 'logistra-robots');
        $logger->info("Try sending order: " . $orderId . " productId: " . $productId . " transportAgreementId " . $transportAgreementId, $context);

        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-logistra-robots-helpers.php';
        $required_options_check = Logistra_Robots_Helpers::check_required_options($orderId, $productId);
        if ($required_options_check['error']) {
            wp_send_json_success(array(
                'orderId' => $orderId,
                'error' => true,
                'messages' => array($required_options_check['message'])
            ));
        }

        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-logistra-robots-order.php';

        $logistra_order = new Logistra_Robots_Order($orderId);
        $skip_services = $productId == null ? null : true;
        $consignmentsArray = $logistra_order->order_to_consignments(array(
            "productId" => $productId,
            "transportAgreementId" => $transportAgreementId,
            "packages" => $packages,
            "skipServices" => $skip_services,
            "weight" => $weight,
            "length" => $length,
            "width" => $width,
            "height" => $height,
        ));
        if (empty($consignmentsArray)) {
            wp_send_json_success(array(
                'orderId' => $orderId,
                'error' => true,
                'messages' => array_merge(array("Ingen fraktmetode på ordre eller feil ved forsendelse overstyring."), $logistra_order->messages)
            ));
        }

        foreach ($consignmentsArray as $consignments) {
            $createConsignmentResponse = Logistra_Robots_Backend::create_consignments($consignments);
            $logistra_order->handle_created_consignments($createConsignmentResponse);
        }


        $logger->info("send order error " . wc_print_r($logistra_order->error, true), $context);
        $logger->info("send order message : " . wc_print_r($logistra_order->messages, true), $context);

        wp_send_json_success(array(
            'error' => $logistra_order->error,
            'messages' => $logistra_order->messages
        ));
    }

    public static function logistra_return_consignments_for_order()
    {
        self::check_security();
        $orderId = sanitize_text_field($_REQUEST['orderId']);
        $productId = sanitize_text_field($_REQUEST['productId']);
        $transportAgreementId = sanitize_text_field($_REQUEST['transportAgreementId']);

        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-logistra-robots-order.php';

        $logistra_order = new Logistra_Robots_Order($orderId);
        $consignmentsArray = $logistra_order->order_to_consignments(array(
            "productId" => $productId,
            "transportAgreementId" => $transportAgreementId,
            "skipServices" => true,
        ));

        if (empty($consignmentsArray)) {
            wp_send_json_success(array(
                'orderId' => $orderId,
                'error' => true,
                'messages' => array_merge(array("Noe feil med parametre for opprettelse av retur forsendelse."), $logistra_order->messages)
            ));
        }

        foreach ($consignmentsArray as $consignments) {
            $createConsignmentResponse = Logistra_Robots_Backend::create_consignments($consignments);
            $logistra_order->handle_created_consignments($createConsignmentResponse);
        }


        wp_send_json_success(array(
            'orderId' => $orderId,
            'error' => $logistra_order->error,
            'messages' => $logistra_order->messages
        ));
    }

    public static function logistra_get_printers()
    {
        self::check_security();

        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-logistra-robots-cargonizer.php';

        $logistra_cargonizer = new Logistra_Robots_Cargonizer();

        $printers = $logistra_cargonizer->get_printers();

        wp_send_json_success(array(
            'printers' => $printers
        ));
    }

    public static function logistra_get_shipping_methods_all()
    {
        self::check_security();
        $shippingMethodDTO = array();
        if (wc_shipping_enabled()) {
            // New version of woocommerce shipping methods
            $shipping_methodping_zones = new WC_Shipping_Zones;
            $zones = $shipping_methodping_zones::get_zones();
            if (!empty($zones)) {
                foreach ($zones as $zone) {
                    foreach ($zone['shipping_methods'] as $shipping_method) {
                        // Only table-rate
                        if (get_option('logistra-robots-use-table-rate-integration') == "yes" && $shipping_method->id === "table_rate") {
                            $rates = $shipping_method->get_shipping_rates();
                            foreach ($rates as $rate) {
                                if ($rate->rate_label == "") {
                                    break;
                                }
                                $instance_id = $shipping_method->instance_id . '#' . $rate->rate_label;
                                array_push($shippingMethodDTO, array(
                                    'name' => $zone['zone_name'] . ' - ' . $shipping_method->title . ' # ' . $rate->rate_label,
                                    'id' => $shipping_method->id . ':' . $instance_id,
                                    'rel' => get_option('logistra-robots-shipping-relation-identifier' . $shipping_method->id . ':' . $instance_id, null),
                                    'transportAgreementId' => get_option('logistra-robots-shipping-relation-transport-agreement-' . $shipping_method->id . ':' . $instance_id, null),
                                    'transportServices' => get_option('logistra-robots-shipping-relation-services-' . $shipping_method->id . ':' . $instance_id, null),
                                    'selectedPrinter' => get_option('logistra-robots-shipping-relation-selected-printer-' . $shipping_method->id . ':' . $instance_id, null),
                                    'selectedPrintTime' => get_option('logistra-robots-shipping-relation-selected-print-time-' . $shipping_method->id . ':' . $instance_id, null),
                                    'selectedTransferTime' => get_option('logistra-robots-shipping-relation-selected-transfer-time-' . $shipping_method->id . ':' . $instance_id, null),
                                ));
                            }
                            continue;
                        } else {
                            array_push($shippingMethodDTO, array(
                                'name' => $zone['zone_name'] . ' - ' . $shipping_method->title,
                                'id' => $shipping_method->id . ':' . $shipping_method->instance_id,
                                'rel' => get_option('logistra-robots-shipping-relation-identifier' . $shipping_method->id . ':' . $shipping_method->instance_id, null),
                                'transportAgreementId' => get_option('logistra-robots-shipping-relation-transport-agreement-' . $shipping_method->id . ':' . $shipping_method->instance_id, null),
                                'transportServices' => get_option('logistra-robots-shipping-relation-services-' . $shipping_method->id . ':' . $shipping_method->instance_id, null),
                                'selectedPrinter' => get_option('logistra-robots-shipping-relation-selected-printer-' . $shipping_method->id . ':' . $shipping_method->instance_id, null),
                                'selectedPrintTime' => get_option('logistra-robots-shipping-relation-selected-print-time-' . $shipping_method->id . ':' . $shipping_method->instance_id, null),
                                'selectedTransferTime' => get_option('logistra-robots-shipping-relation-selected-transfer-time-' . $shipping_method->id . ':' . $shipping_method->instance_id, null),
                            ));
                        }
                    }
                }
            }
        } else {
            // Old version of shipping method
            $shipping_methods = WC()->shipping->load_shipping_methods();
            foreach ($shipping_methods as $shipping_method) {
                if (isset($shipping_method->enabled) && 'yes' === $shipping_method->enabled) {
                    array_push($shippingMethodDTO, array(
                        'name' => $shipping_method->method_title . ' - ' . $shipping_method->title,
                        'id' => $shipping_method->id,
                        'rel' => get_option('logistra-robots-shipping-relation-identifier' . $shipping_method->id, null),
                        'transportAgreementId' => get_option('logistra-robots-shipping-relation-transport-agreement-' . $shipping_method->id, null),
                        'transportServices' => get_option('logistra-robots-shipping-relation-services-' . $shipping_method->id, null),
                        'selectedPrinter' => get_option('logistra-robots-shipping-relation-selected-printer-' . $shipping_method->id, null),
                        'selectedPrintTime' =>  get_option('logistra-robots-shipping-relation-selected-print-time-' . $shipping_method->id, null),
                        'selectedTransferTime' =>  get_option('logistra-robots-shipping-relation-selected-transfer-time-' . $shipping_method->id, null),
                    ));
                }
            }
        };

        wp_send_json_success(array(
            'message' => "Get shipping methods",
            'shippingMethods' => $shippingMethodDTO
        ));
    }

    public static function logistra_update_shipping_method_relation()
    {
        self::check_security();
        $shippingMethod = sanitize_text_field($_REQUEST['shippingMethod']);
        $transportProductIdentifier = sanitize_text_field($_REQUEST['transportProductIdentifier']);
        $transportProductCarrier = sanitize_text_field($_REQUEST['transportProductCarrier']);
        $transportAgreementId = sanitize_text_field($_REQUEST['transportAgreementId']);
        // $transportProductId = sanitize_text_field($_REQUEST['transportProductId']);
        $transportServicesCommaDelimted = sanitize_text_field($_REQUEST['transportServices']);
        $selectedPrinter = sanitize_text_field($_REQUEST['selectedPrinter']);
        $selectedPrintTime = sanitize_text_field($_REQUEST['selectedPrintTime']);
        $selectedTransferTime = sanitize_text_field($_REQUEST['selectedTransferTime']);

        $message = "";

        if ($transportProductIdentifier === "null") {
            $transportProductIdentifier = null;
        };
        if ($transportProductCarrier === "null") {
            $transportProductCarrier = null;
        };
        if ($transportAgreementId === "null") {
            $transportAgreementId = null;
        };
        if ($selectedPrinter === "null") {
            $selectedPrinter = null;
        };
        if ($selectedPrintTime === "null") {
            $selectedPrintTime = null;
        };
        if ($selectedTransferTime === "null") {
            $selectedTransferTime = null;
        };

        !update_option('logistra-robots-shipping-relation-identifier' .  $shippingMethod,  $transportProductIdentifier, '', false);
        !update_option('logistra-robots-shipping-relation-carrier' .  $shippingMethod,  $transportProductCarrier, '', false);
        // !update_option('logistra-robots-shipping-relation-product-id-' .  $shippingMethod,  $transportProductId, '', false);
        !update_option('logistra-robots-shipping-relation-transport-agreement-' .  $shippingMethod,  $transportAgreementId, '', false);
        !update_option('logistra-robots-shipping-relation-services-' .  $shippingMethod,  $transportServicesCommaDelimted, '', false);
        !update_option('logistra-robots-shipping-relation-selected-printer-' .  $shippingMethod,  $selectedPrinter, '', false);
        !update_option('logistra-robots-shipping-relation-selected-print-time-' .  $shippingMethod,  $selectedPrintTime, '', false);
        !update_option('logistra-robots-shipping-relation-selected-transfer-time-' .  $shippingMethod,  $selectedTransferTime, '', false);


        return self::logistra_get_shipping_methods_all();
    }

    /* PUBLIC METHODS */



    public static function logistra_get_service_partners()
    {
        self::check_security();
        $shipping_method = sanitize_text_field($_REQUEST['shippingMethod']);
        $country = WC()->customer->get_shipping_country();
        $postcode = WC()->customer->get_shipping_postcode();

        // If cant get this dont get service partners
        if ($country === null) {
            wp_send_json_error(array(
                "message" => "Trenger å vite hvilket land pakken skal til for å hente utleveringssteder."
            ));
        }
        if ($postcode === null) {
            wp_send_json_error(array(
                "message" => "Trenger å vite hvilken postkode pakken skal til for å hente utleveringssteder."
            ));
        }

        $transient_name = 'logistra-robots-service-partner-response-' . $country . '-' . $postcode . '-' . $shipping_method;
        $transient = get_transient($transient_name);
        if (false === $transient || "" === $transient) {
            require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-logistra-robots-helpers.php';
            $shipping = Logistra_Robots_Helpers::get_method_and_instance_from_shipping_method($shipping_method);
            $carrier = Logistra_Robots_Helpers::get_carrier_for_shipment_method($shipping['method_id'], $shipping['instance_id']);
            $transport_product_indetifier = Logistra_Robots_Helpers::get_transport_product_indetifier_for_shipping_method($shipping['method_id'], $shipping['instance_id']);
            $transport_product_agreement = Logistra_Robots_Helpers::get_transport_agreement_for_shipping_method($transport_product_indetifier);
            if ($transport_product_agreement["service_partner_possible"]) {
                require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-logistra-robots-cargonizer.php';

                $logistra_cargonizer = new Logistra_Robots_Cargonizer();

                $service_partners = $logistra_cargonizer->get_service_partners($country, $postcode, $carrier);
                $service_partners['carrier'] = $carrier;

                $response = array_merge($service_partners, array(
                    "status" => 1,
                    "message" => "Hentet utleveringsteder."
                ));
            } else {
                $response = array(
                    "status" => 2,
                    "message" => "Fraktmetode og transport trenger ikke utleveringsted."
                );
            }
            set_transient($transient_name, $response, 7 * DAY_IN_SECONDS);
        } else {
            $response = $transient;
        }
        wp_send_json_success($response);
    }

    // I dont think this is used
    public static function logistra_get_shipping_method()
    {
        self::check_security();
        $orderId = sanitize_text_field($_REQUEST['orderId']);

        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-logistra-robots-order.php';

        $logistra_order = new Logistra_Robots_Order($orderId);
        $consignmentsArray = $logistra_order->order_to_consignments(array());

        if (empty($consignmentsArray)) {
            wp_send_json_success(array(
                'orderId' => $orderId,
                'error' => true,
                'messages' => array_merge(array("Noe feil med parametre for opprettelse av forsendelse."), $logistra_order->messages)
            ));
        }

        foreach ($consignmentsArray as $consignments) {
            $createConsignmentResponse = Logistra_Robots_Backend::create_consignments($consignments);
            $logistra_order->handle_created_consignments($createConsignmentResponse);
        }


        wp_send_json_success(array(
            'orderId' => $orderId,
            'error' => $logistra_order->error,
            'messages' => $logistra_order->messages
        ));
    }

    /* PRIVATE METHODS */

    private static function check_security()
    {
        check_ajax_referer('randomTextForLogistraIntegration', 'security');
    }

    private static function check_field($requestVariableName)
    {
        $variable = sanitize_text_field($_REQUEST[$requestVariableName]);
        if ($variable === "null" || $variable === "undefined" || $variable === "") {
            $variable = null;
        };
        if ($variable === "true") {
            $variable = 'yes';
        };
        if ($variable === "false") {
            $variable = 'no';
        };
        return $variable;
    }
} // End class AJAX
new Ajax();
