<?php

/**
 * Fired during plugin activation
 *
 * @link       https://wildrobot.app/logistra
 * @since      1.0.0
 *
 * @package    Logistra_Robots
 * @subpackage Logistra_Robots/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    Logistra_Robots
 * @subpackage Logistra_Robots/includes
 * @author     Robertosnap <robertosnap+spam@pm.me>
 */
class Logistra_Robots_Backend
{
    private static $logistra_backend_url_prod = "https://logistra-robots-backend.herokuapp.com";
    // private static $logistra_backend_url_docker = "https://logistra-robots-backend-test.now.sh"; // Use ngrok if you are running the server on localhost
    private static $logistra_backend_url_docker = "https://logistra-robots-backend-stage.herokuapp.com"; // Use ngrok if you are running the server on localhost
    // private static $logistra_backend_url_docker = "http://lrobotsdemo.serverless.social"; // Use ngrok if you are running the server on localhost
    private static $logistra_backend_url_test = "http://localhost:3000"; // practically never used
    private static $endpoint_service_partner = '/cargonizer/service_partners';
    private static $endpoint_consignments = '/cargonizer/consignments';

    public static function get_service_partners($country, $postcode, $carrier)
    {
        $response = wp_remote_get(add_query_arg(array(
            'country'  => esc_html($country),
            'postcode' => esc_html($postcode),
            'carrier' => esc_html($carrier),
        ), self::logistra_backend_url() . self::$endpoint_service_partner), array(
            'headers'     => array(
                'Authorization' => "bearer " . get_option('logistra-robots-token', null),
            ),
        ));
        $repsponseBody = wp_remote_retrieve_body($response);
        $responeObject = json_decode($repsponseBody);
        if (empty($responeObject->servicePartners)) {
            $responeObject->servicePartners[0] = self::get_fallback_service_partner();
        }
        return $responeObject;
    }

    public static function create_consignments($consignments)
    {
        $body = json_encode($consignments);
        $response = wp_remote_post(self::logistra_backend_url() . self::$endpoint_consignments, array(
            'headers' => array(
                'Authorization' => "bearer " . get_option('logistra-robots-token', null),
                "Content-type"        => "application/json",
            ),
            'body'    => $body,
        ));

        $repsponseBody = wp_remote_retrieve_body($response);
        $responeObject = json_decode($repsponseBody);

        return $responeObject;
    }

    private static function get_fallback_service_partner()
    {
        return (object) [
            "customer-number" => \esc_html('00098287279'),
            'number'          => \esc_html('0413898'),
            'name'            => \esc_html('Postens Godssenter'),
            'country'         => \esc_html('NO'),
            'postcode'        => \esc_html('0068'),
            'city'            => \esc_html('Oslo'),
        ];
    }

    private static function is_test()
    {
        if (strpos(get_permalink(wc_get_page_id('shop')), "localhost") !== false) {
            return true;
        };
        return false;
    }

    private static function is_docker()
    {
        if (strpos(get_permalink(wc_get_page_id('shop')), "192.168.99.100") !== false || strpos(get_permalink(wc_get_page_id('shop')), "172.18.0.3") !== false) {
            return true;
        };
        return false;
    }

    public static function logistra_backend_url()
    {
        if (self::is_test()) {
            return self::$logistra_backend_url_test;
        }
        if (self::is_docker()) {
            return self::$logistra_backend_url_docker;
        }
        return self::$logistra_backend_url_prod;
    }
}
