<?php

class Logistra_Robots_Cargonizer
{
    private static $logistra_cargonizer_url_prod = "https://cargonizer.no"; // todo: Fix for prod
    private static $logistra_cargonizer_url_docker = "https://cargonizer.no"; // Use ngrok if you are running the server on localhost
    private static $logistra_cargonizer_url_test = "https://sandbox.cargonizer.no";
    private static $endpoint_service_partner = '/service_partners.xml';
    private static $endpoint_consignments = '/consignments.xml';
    private static $endpoint_printers = '/printers.xml';

    public function __construct()
    {
        add_filter('http_request_args', array($this, 'curl_timeout'), 99, 1);
    }
    public function curl_timeout($r)
    {
        $r['timeout'] = 5;
        return $r;
    }

    public static function get_service_partners($country, $postcode, $carrier)
    {
        $transient_name = 'logistra-robots-service-partners-' . $country . '-' . $postcode . '-' . $carrier;
        if (false === ($transient = get_transient($transient_name))) {
            $api_key = get_option('logistra-robots-apikey', null);
            $sender_id = get_option('logistra-robots-senderid', null);
            $response = wp_remote_get(add_query_arg(array(
                'country'  => esc_html($country),
                'postcode' => esc_html($postcode),
                'carrier' => esc_html($carrier),
            ), self::logistra_cargonizer_url() . self::$endpoint_service_partner), array(
                'headers'     => array(
                    'X-Cargonizer-Key' => $api_key,
                    'X-Cargonizer-Sender' => $sender_id,
                ),
            ));

            $repsponseBody = wp_remote_retrieve_body($response);
            $xml = simplexml_load_string($repsponseBody);
            $json = json_encode($xml);
            $data = json_decode($json, TRUE);
            if (array_key_exists('number', $data["service-partners"]["service-partner"])) {
                $data["servicePartners"] = array(0 => $data["service-partners"]["service-partner"]);
            } else {
                $data["servicePartners"] = $data["service-partners"]["service-partner"];
            }
            unset($data["service-partners"]);

            set_transient($transient_name, $data, 7 * DAY_IN_SECONDS);
        } else {
            $data = $transient;
        }
        return $data;
    }

    public static function get_printers()
    {
        $sendersCommaDelimted = get_option('logistra-robots-senders', null);
        $senders = explode(',', $sendersCommaDelimted);
        $printers = array();
        foreach ($senders as $senderId) {
            $response = wp_remote_get(self::logistra_cargonizer_url() . self::$endpoint_printers, array(
                'headers'     => array(
                    'X-Cargonizer-Key' => get_option('logistra-robots-apikey', null),
                    'X-Cargonizer-Sender' => $senderId,
                ),
            ));
            $repsponseBody = wp_remote_retrieve_body($response);

            $xml = simplexml_load_string($repsponseBody);
            $json = json_encode($xml);
            $array = json_decode($json, TRUE);
            // If we get an empty array we must handle this like this
            if (!array_key_exists("printer", $array)) {
                continue;
            }
            if (array_key_exists('id', $array["printer"])) {
                $printers = array_merge($printers, array(0 => $array["printer"]));
            } else {
                $printers = array_merge($printers, $array["printer"]);
            }
        }
        $unique_printers = self::better_array_unique($printers);
        update_option('logistra-robots-printers',  wp_json_encode($unique_printers));
        return $unique_printers;
    }

    private static function get_fallback_service_partner()
    {
        return (object) [
            "customer-number" => \esc_html('00098287279'),
            'number'          => \esc_html('0413898'),
            'name'            => \esc_html('Postens Godssenter'),
            'country'         => \esc_html('NO'),
            'postcode'        => \esc_html('0068'),
            'city'            => \esc_html('Oslo'),
        ];
    }

    private static function is_test()
    {
        if (strpos(get_permalink(wc_get_page_id('shop')), "localhost") !== false) {
            return true;
        };
        return false;
    }

    private static function is_docker()
    {
        if (strpos(get_permalink(wc_get_page_id('shop')), "192.168.99.100") !== false) {
            return true;
        };
        return false;
    }

    public static function logistra_cargonizer_url()
    {
        if (self::is_test()) {
            return self::$logistra_cargonizer_url_test;
        }
        if (self::is_docker()) {
            return self::$logistra_cargonizer_url_docker;
        }
        return self::$logistra_cargonizer_url_prod;
    }

    private static function better_array_unique($array, $keep_key_assoc = false)
    {
        $duplicate_keys = array();
        $tmp = array();

        foreach ($array as $key => $val) {
            // convert objects to arrays, in_array() does not support objects
            if (is_object($val))
                $val = (array) $val;

            if (!in_array($val, $tmp))
                $tmp[] = $val;
            else
                $duplicate_keys[] = $key;
        }

        foreach ($duplicate_keys as $key)
            unset($array[$key]);

        return $keep_key_assoc ? $array : array_values($array);
    }
}
