<?php

/**
 * Fired during plugin activation
 *
 * @link       https://wildrobot.app/logistra
 * @since      1.0.0
 *
 * @package    Logistra_Robots
 * @subpackage Logistra_Robots/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    Logistra_Robots
 * @subpackage Logistra_Robots/includes
 * @author     Robertosnap <robertosnap+spam@pm.me>
 */
class Logistra_Robots_Helpers
{

    public $order;
    public $error = false;
    public $messages = array();

    public static function init()
    {
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-logistra-robots-backend.php';
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-logistra-robots-cargonizer.php';
    }

    public static function get_shipping_method_values($shp)
    {
        $shipping  = self::get_method_and_instance_from_shipping_method($shp);

        $shippingMethodValues = array(
            // 'product' =>  self::get_transport_product_for_shipping_method($shipping['method_id'], $shipping['instance_id']),
            'productIdentifier' =>  self::get_transport_product_indetifier_for_shipping_method($shipping['method_id'], $shipping['instance_id']),
            'productCarrier' =>  self::get_transport_product_carrier_for_shipping_method($shipping['method_id'], $shipping['instance_id']),
            // 'productId' =>  self::get_transport_product_id_for_shipping_method($shipping['method_id'], $shipping['instance_id']),
            'transportAgreementId' =>  self::get_transport_agreement_id_for_shipping_method($shipping['method_id'], $shipping['instance_id']),
            'services' =>  self::get_transport_services_for_shipping_method($shipping['method_id'], $shipping['instance_id']),
            'carrier' => self::get_carrier_for_shipment_method($shipping['method_id'], $shipping['instance_id']),
            'printer' => self::get_printer_selected_for_shipping_method($shipping['method_id'], $shipping['instance_id']),
            'selectedPrintTime' => self::get_print_time_for_shipping_method($shipping['method_id'], $shipping['instance_id']),
            'selectedTransferTime' => self::get_transfer_time_for_shipping_method($shipping['method_id'], $shipping['instance_id']),
        );

        $shippingMethodValues['transportAgreement'] = self::get_transport_agreement_for_shipping_method($shippingMethodValues['productIdentifier']);

        return $shippingMethodValues;
    }

    public static function get_method_and_instance_from_shipping_method($shp)
    {
        // if $shp is of type shipping method instance
        if ($shp instanceof WC_Order_Item_Shipping) {
            $method_id = $shp->get_method_id();
            if (get_option('logistra-robots-use-table-rate-integration') == "yes" && $method_id === "table_rate") {
                $instance_id = $shp->get_instance_id() . "#" . $shp->get_method_title();
            } else {
                $instance_id = $shp->get_instance_id();
            }
        } else {
            $shp_strings = explode(':', $shp);
            $method_id = $shp_strings[0];
            $instance_id = $shp_strings[1];
        }
        return array(
            'method_id' => $method_id,
            'instance_id' => $instance_id,
        );
    }


    public static function get_pickuppoints($shp, $country, $postcode)
    {
        $shipping = Logistra_Robots_Helpers::get_method_and_instance_from_shipping_method($shp);
        $servicePartnersObject = Logistra_Robots_Cargonizer::get_service_partners(
            $country,
            $postcode,
            self::get_carrier_for_shipment_method(
                $shipping['method_id'],
                $shipping['instance_id']
            )
        );
        return $servicePartnersObject["servicePartners"];
    }

    public static function get_spesific_or_closest_pickuppoint($shp, $country, $postcode, $number)
    {
        $servicePartners = self::get_pickuppoints($shp, $country, $postcode);
        $bestMatch = $servicePartners[0];
        foreach ($servicePartners as $key => $servicePartner) {
            if (intval($number) == intval($servicePartner['number'])) {
                $bestMatch = $servicePartner;
            }
        }
        return $bestMatch;
    }

    public static function get_pickuppoints2($carrier, $country, $postcode)
    {
        $servicePartnersObject = Logistra_Robots_Cargonizer::get_service_partners(
            $country,
            $postcode,
            $carrier
        );
        return $servicePartnersObject["servicePartners"];
    }

    public static function get_spesific_or_closest_pickuppoint2($carrier, $country, $postcode, $choosenServiceParnterNumber)
    {
        $servicePartners = self::get_pickuppoints2($carrier, $country, $postcode);
        $bestMatch = $servicePartners[0];
        foreach ($servicePartners as $key => $servicePartner) {
            if (intval($choosenServiceParnterNumber) == intval($servicePartner['number'])) {
                $bestMatch = $servicePartner;
            }
        }
        return $bestMatch;
    }

    public static function check_required_options($orderId, $check_transport_product = null)
    {
        $message = "";
        $error = false;

        if (get_option('logistra-robots-apikey', null) === null || get_option('logistra-robots-apikey', null) === "" || get_option('logistra-robots-apikey', null) === "null") {
            $message .= __('Vennligst sett API nøkkel før du sender ordre. ');
            $error = true;
        }
        if (get_option('logistra-robots-senderid', null) === null || get_option('logistra-robots-senderid', null) === "" || get_option('logistra-robots-senderid', null) === "null") {
            $message .= __('Vennligst sett avsender før du sender ordre. ');
            $error = true;
        }

        // if (get_option('logistra-robots-printer-default', null) === null || get_option('logistra-robots-printer-default', null) === "" || get_option('logistra-robots-printer-default', null) === "null") {
        //     $message .= __('Vennligst sett standard etiketteskriver før du sender ordre. ');
        //     $error = true;
        // }

        $order = wc_get_order($orderId);
        // Maybe check transport product for order
        if ($check_transport_product === null) {
            foreach ($order->get_shipping_methods() as $shp) {
                $shipping  = self::get_method_and_instance_from_shipping_method($shp);
                $transport_product = self::get_transport_product_indetifier_for_shipping_method($shipping['method_id'], $shipping['instance_id']);
                if ($transport_product === null || $transport_product === "" || $transport_product === "null") {
                    $message .= __('Frakmetoden til denne ordren har ingen transport produkt fra Logistra Cargonizer.');
                    $error = true;
                }
            }
        }

        // Check that we have a printer
        foreach ($order->get_shipping_methods() as $shp) {
            $shippingMethodValues = self::get_shipping_method_values($shp);
            $hasSpesificPrinter = ($shippingMethodValues['printer'] !== null && $shippingMethodValues['printer'] !== "");
            $printer = $hasSpesificPrinter ? $shippingMethodValues['printer'] : get_option('logistra-robots-printer-default', null);
            if ($printer === "" || $printer === null || $printer ===  "null") {
                $message .= __('Ingen spesifikk printer satt for fraktmetoden og ingen standard printer satt i instillinger.');
                $error = true;
            }
        }

        return array(
            'message' => $message,
            'error' => $error,
        );
    }

    public static function clear_all_options()
    {
        update_option('logistra-robots-senderid', null, '', false);
        update_option('logistra-robots-senders', null, '', false);
        update_option('logistra-robots-sender-name', null, '', false);
        update_option('logistra-robots-printer-default', null, '', false);
        update_option('logistra-robots-transport-agreements', null, '', false);
        self::clear_all_shipping_method_relations();
    }

    public static function clear_all_shipping_method_relations()
    {
        // Clear all options for shipping methods
        if (wc_shipping_enabled()) {
            // New version of woocommerce shipping methods
            $shpping_zones = new WC_Shipping_Zones;
            $zones = $shpping_zones::get_zones();
            if (!empty($zones)) {
                foreach ($zones as $zone) {
                    foreach ($zone['shipping_methods'] as $shipping_method) {
                        if ($shipping_method->id === "table_rate") {
                            $rates = $shipping_method->get_shipping_rates();
                            foreach ($rates as $rate) {
                                if ($rate->rate_label == "") {
                                    break;
                                }
                                $instance_id = $shipping_method->instance_id . '#' . $rate->rate_label;
                                update_option('logistra-robots-shipping-relation-' . $shipping_method->id . ':' . $instance_id, null, '', false);
                                update_option('logistra-robots-shipping-relation-transport-agreement-' . $shipping_method->id . ':' . $instance_id, null, '', false);
                                update_option('logistra-robots-shipping-relation-services-' . $shipping_method->id . ':' . $instance_id, null, '', false);
                                update_option('logistra-robots-shipping-relation-selected-printer-' . $shipping_method->id . ':' . $instance_id, null, '', false);
                                update_option('logistra-robots-shipping-relation-selected-print-time-' . $shipping_method->id . ':' . $instance_id, null, '', false);
                                update_option('logistra-robots-shipping-relation-selected-transfer-time-' . $shipping_method->id . ':' . $instance_id, null, '', false);
                            }
                            continue;
                        }
                        $instance_id = $shipping_method->instance_id;
                        update_option('logistra-robots-shipping-relation-' . $shipping_method->id . ':' . $instance_id, null, '', false);
                        update_option('logistra-robots-shipping-relation-transport-agreement-' . $shipping_method->id . ':' . $instance_id, null, '', false);
                        update_option('logistra-robots-shipping-relation-services-' . $shipping_method->id . ':' . $instance_id, null, '', false);
                        update_option('logistra-robots-shipping-relation-selected-printer-' . $shipping_method->id . ':' . $instance_id, null, '', false);
                        update_option('logistra-robots-shipping-relation-selected-print-time-' . $shipping_method->id . ':' . $instance_id, null, '', false);
                        update_option('logistra-robots-shipping-relation-selected-transfer-time-' . $shipping_method->id . ':' . $instance_id, null, '', false);
                    }
                }
            }
        } else {
            // Old version of shipping method
            $shipping_methods = WC()->shipping->load_shipping_methods();
            foreach ($shipping_methods as $shipping_method) {
                if (isset($shipping_method->enabled) && 'yes' === $shipping_method->enabled) {
                    get_option('logistra-robots-shipping-relation-' . $shipping_method->id, null, '', false);
                    get_option('logistra-robots-shipping-relation-transport-agreement-' . $shipping_method->id, null, '', false);
                    get_option('logistra-robots-shipping-relation-services-' . $shipping_method->id, null, '', false);
                    get_option('logistra-robots-shipping-relation-selected-printer-' . $shipping_method->id, null, '', false);
                    get_option('logistra-robots-shipping-relation-selected-print-time-' . $shipping_method->id, null, '', false);
                    get_option('logistra-robots-shipping-relation-selected-transfer-time-' . $shipping_method->id, null, '', false);
                }
            }
        };
    }

    public static function get_closest_pickuppoint($shp, $country, $postcode)
    {
        return self::get_pickuppoints($shp, $country, $postcode)[0];
    }

    public static function get_transport_agreement_for_shipping_method($transportProduct)
    {
        // $transport_agreements = json_decode(json_decode("\"" . get_option('logistra-robots-transport-agreements', json_encode(new stdClass())) . "\""));
        $transport_agreements_encoded_from_js = get_option('logistra-robots-transport-agreements', wp_json_encode(new stdClass()));
        // $js_to_php_hack = "\"" . $transport_agreements_encoded_from_js . "\"";
        // $transport_agreements = json_decode(json_decode($js_to_php_hack, false,  50000), false, 50000);
        // $test = $transport_agreements[0];    
        $js_to_php_hack = stripslashes($transport_agreements_encoded_from_js);
        $transport_agreements = json_decode($js_to_php_hack, true);
        // $test = $transport_agreements[1]["identifier"];
        // $test2 = $transport_agreements[1]["services"];
        $currentTransportAgreement = null;
        foreach ($transport_agreements as $transport_agreement) {
            if ($transportProduct === $transport_agreement["identifier"]) {
                $currentTransportAgreement = $transport_agreement;
                break;
            }
        }
        // if ($currentTransportAgreement === null) {
        //     throw new Error("Could not find transport agreement");
        // }
        return $currentTransportAgreement;
    }

    public static function get_carrier_for_shipment_method($shipping_method_id, $shipping_method_instance_id)
    {
        return self::get_transport_product_carrier_for_shipping_method($shipping_method_id, $shipping_method_instance_id);
    }

    public static function get_transport_agreement_id_for_shipping_method($shipping_method_id, $shipping_method_instance_id)
    {
        $option = get_option('logistra-robots-shipping-relation-transport-agreement-' . $shipping_method_id . ':' . $shipping_method_instance_id, null);
        return $option === "" ? null : $option;
    }

    public static function get_transport_product_id_for_shipping_method($shipping_method_id, $shipping_method_instance_id)
    {
        $option = get_option('logistra-robots-shipping-relation-product-id-' . $shipping_method_id . ':' . $shipping_method_instance_id, null);
        return $option === "" ? null : $option;
    }

    // public static function get_transport_product_for_shipping_method($shipping_method_id, $shipping_method_instance_id)
    // {
    //     $option = get_option('logistra-robots-shipping-relation-' . $shipping_method_id . ':' . $shipping_method_instance_id, null);
    //     return $option;
    // }
    public static function get_transport_product_indetifier_for_shipping_method($shipping_method_id, $shipping_method_instance_id)
    {
        $option = get_option('logistra-robots-shipping-relation-identifier' . $shipping_method_id . ':' . $shipping_method_instance_id, null);
        return $option;
    }

    public static function get_transport_product_carrier_for_shipping_method($shipping_method_id, $shipping_method_instance_id)
    {
        $option = get_option('logistra-robots-shipping-relation-carrier' . $shipping_method_id . ':' . $shipping_method_instance_id, null);
        return $option;
    }

    public static function get_transport_services_for_shipping_method($shipping_method_id, $shipping_method_instance_id)
    {
        $option = get_option('logistra-robots-shipping-relation-services-' . $shipping_method_id . ':' . $shipping_method_instance_id, null);
        return explode(',', $option);
    }

    public static function get_printer_selected_for_shipping_method($shipping_method_id, $shipping_method_instance_id)
    {
        $option = get_option('logistra-robots-shipping-relation-selected-printer-' . $shipping_method_id . ':' . $shipping_method_instance_id, null);
        return $option === "" ? null : $option;
    }

    public static function get_print_time_for_shipping_method($shipping_method_id, $shipping_method_instance_id)
    {
        $option = get_option('logistra-robots-shipping-relation-selected-print-time-' . $shipping_method_id . ':' . $shipping_method_instance_id, null);
        return $option === "" ? null : $option;
    }

    public static function get_transfer_time_for_shipping_method($shipping_method_id, $shipping_method_instance_id)
    {
        $option = get_option('logistra-robots-shipping-relation-selected-transfer-time-' . $shipping_method_id . ':' . $shipping_method_instance_id, null);
        return $option === "" ? null : $option;
    }
}
Logistra_Robots_Helpers::init();
