<?php
if (!defined('ABSPATH')) {
    exit;
}

add_action('woocommerce_shipping_init', 'logistra_robots_shipping_method_init');
add_action('woocommerce_shipping_methods', 'add_logistra_robots_shipping_method');


function add_logistra_robots_shipping_method($methods)
{
    $methods['logistra_robots_shipping_method'] = 'WC_Logistra_Robots_Shipping_Method';
    return $methods;
}

function logistra_robots_shipping_method_init()
{
    if (!class_exists('WC_Logistra_Robots_Shipping_Method')) {
        class WC_Logistra_Robots_Shipping_Method extends WC_Shipping_Method
        {
            /**
             * Constructor for your shipping class
             *
             * @access public
             * @return void
             */
            public function __construct($instance_id = 0)
            {
                $this->id                       = 'logistra_robots_shipping_method';
                $this->instance_id              = absint($instance_id);
                $this->method_title             = __('Vekt og gratisgrense (Logistra Wildrobot)', 'logistra_robots');
                $this->method_description       = __('Lag fraktmetoder med parametre for vekt og gratis frakt.', 'logistra_robots'); // 
                // $this->enabled                  = "yes"; // This can be added as an setting but for this example its forced enabled

                $this->supports           = array(
                    'shipping-zones',
                    'instance-settings',
                    'instance-settings-modal',
                );

                $this->init();
            }

            /**
             * Init your settings
             *
             * @access public
             * @return void
             */
            function init()
            {
                // Load the settings API
                $this->instance_form_fields = $this->settings(); // This is part of the settings API. Override the method to add your own settings
                // $this->init_settings(); // This is part of the settings API. Loads settings you previously init.

                $this->title        = $this->get_option('title');
                // $this->description        = $this->get_option('description');
                $this->tax_status        = $this->get_option('tax_status');
                $this->cost        = $this->get_option('cost');
                $this->weight_controlled        = $this->get_option('weight_controlled');
                $this->from_weight        = $this->get_option('from_weight');
                $this->too_weight        = $this->get_option('too_weight');
                $this->free_treshhold        = $this->get_option('free_treshhold');
                $this->free_value        = $this->get_option('free_value');
                $this->max_value_treshhold        = $this->get_option('max_value_treshhold');
                $this->max_value        = $this->get_option('max_value');

                // Save settings in admin if you have any defined
                add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
            }

            /**
             * calculate_shipping function.
             *
             * @access public
             * @param mixed $package
             * @return void
             */
            public function calculate_shipping($package = array())
            {
                $has_cost = false;
                $valid = true;
                $rate = array(
                    'id'      => $this->get_rate_id(),
                    'label'   => $this->title,
                    'cost'    => 0,
                    'package' => $package,
                );

                if ($this->cost !== '') {
                    $has_cost    = true;
                    $rate['cost'] = $this->cost;
                }

                $weight = $this->get_package_weight($package);
                if ($this->weight_controlled === "yes") {
                    if ($weight >= $this->from_weight && $weight < $this->too_weight) {
                        $has_cost = true;
                    } else {
                        $valid = false;
                    }
                }

                // If shp still valid lets calc value of order
                if ($valid) {
                    // calculate order total
                    $order_total = 0;
                    foreach ($package['contents'] as $product) {

                        $order_total = $order_total + $product['line_subtotal'];
                        // Changed calculation, this should probably be removed
                        // if ($this->tax_status === "taxable") {
                        //     $order_total = $order_total + $product['line_subtotal_tax'];
                        // }
                        $order_total = $order_total + $product['line_subtotal_tax'];
                    }
                    
                    
                    if ($this->free_treshhold === "yes") {
                        $notice = get_option('logistra-robots-free-freight-notice', null) === "yes";
                        $almost_value = get_option('logistra-robots-free-freight-almost-value', 0);

                        if($order_total >= floatval($this->free_value)){
                            if($notice === "yes"){
                                wc_clear_notices();
                                $message = apply_filters('logistra_robots_free_freight_message', sprintf(__('Du har nå gratis frakt på %s', 'logistra-robots'), $rate['label']));
                                wc_add_notice($message, 'success');
                            }
                            $rate['cost'] = 0;
                            $rate['label'] = $rate['label'] . ' ' . apply_filters('logistra_robots_free_label_title_extended', __('(Gratis)', 'logistra-robots'));
                            $has_cost = true;
                        } else if( $notice && $almost_value > 0 &&  $order_total >= floatval($this->free_value) - $almost_value ) {
                            wc_clear_notices();
                            $message = apply_filters('logistra_robots_free_freight_message', sprintf(__('%s kr fra fraktfri levering. <a href="%s">Fortsett å handle</a>', 'logistra-robots'), floatval($this->free_value) - $order_total, wc_get_page_permalink("shop") ));
                            wc_add_notice($message, 'notice');
                        }
                    }

                    if($this->max_value_treshhold === "yes" && $order_total > floatval($this->max_value)){
                            $valid = false;    
                    }
                }

                if ($has_cost && $valid) {
                    $this->add_rate($rate);
                    do_action('woocommerce_' . $this->id . '_shipping_add_rate', $this, $rate);
                }
            }

            public function get_package_weight($package = array())
            {
                $weight    = 0;
                $_pf = new WC_Product_Factory();

                foreach ($package['contents'] as $item) {
                    if ($item['product_id'] > 0) {
                        $_product = $_pf->get_product($item['product_id']);
                        if (!\is_object($_product)) continue; // skip virtual products

                        if (!$_product->is_virtual()) {
                            $product_weight = $_product->get_weight();

                            if ($product_weight) {
                                $weight += $_product->get_weight() * $item['quantity'];
                            }
                        }
                    }
                }
                if ($weight > 0)
                    return round($weight, 3, PHP_ROUND_HALF_DOWN);
                return 0;
            }


            function settings()
            {
                return array(
                    'title' => array(
                        'title' => __('Title', 'logistra-robots'),
                        'type' => 'text',
                        'description' => __('Navnet på fraktmetoden som kunden ser i kassen.', 'logistra-robots'),
                        'default' => __('Vekt og gratisgrense fraktmetode (endre tittel) ', 'logistra-robots')
                    ),
                    // 'description' => array(
                    //     'title' => __('Description', 'logistra-robots'),
                    //     'type' => 'textarea',
                    //     'description' => __('Forklaring på fraktmetoden som kunden ser i kassen.', 'logistra-robots'),
                    //     'default' => __("...", 'logistra-robots')
                    // ),
                    'tax_status' => array(
                        'title'                 => __('MVA', 'logistra-robots'),
                        'type'                  => 'select',
                        'class'                 => 'wc-enhanced-select',
                        'default'               => 'taxable',
                        'options'               => array(
                            'taxable'               => __('Avgift', 'logistra-robots'),
                            'none'                  => _x('Ikke-avgift', 'Tax status', 'logistra-robots')
                        )
                    ),
                    'cost' => array(
                        'title'                 => __('Kostnad', 'logistra-robots'),
                        'type'                  => 'number',
                        'placeholder'           => '',
                        'description'           => "Pris for fraktmetoden.",
                        'default'               => '0',
                        'desc_tip'              => true
                    ),
                    'weight_controlled' => array(
                        'title'                 => __('Vekt styrt', 'logistra-robots'),
                        'type'                  => 'checkbox',
                        'class'                 => '',
                        'css'                   => 'width: 16px; paddding-top: 10px;',
                        'placeholder'           => '',
                        'label'                 => __('Skal frakmetoden styres av vekt på ordren?', 'logistra-robots'),
                        // 'description'           => __('Skal frakmetoden styres av vekt på ordren?', 'logistra-robots'),
                        'default'               => 'no',
                        'desc_tip'              => false,
                        'custom_attributes'     => array(),
                        'disabled'              => false,
                    ),
                    'from_weight' => array(
                        'title'                 => __('Fra vekt', 'logistra-robots'),
                        'type'                  => 'number',
                        'placeholder'           => '',
                        'description'           => 'Fra vekt er inklusiv. Skriver du 1, vil den altså gjelde for et produkt på akkurat 1 kg. Vekt styrt må være aktivert.',
                        'default'               => '0',
                        'desc_tip'              => true
                    ),
                    'too_weight' => array(
                        'title'                 => __('Til vekt', 'logistra-robots'),
                        'type'                  => 'number',
                        'placeholder'           => '',
                        'description'           => "Til vekt er eksklusiv. Skriver du 2, vil den gjelde opp til 2 og IKKE for et produkt på akkurat 2 kg. Vekt styrt må være aktivert.",
                        'default'               => '0',
                        'desc_tip'              => true
                    ),
                    'free_treshhold' => array(
                        'title'                 => __('Gratis frakt terskel', 'logistra-robots'),
                        'type'                  => 'checkbox',
                        'class'                 => '',
                        'css'                   => 'width: 16px; paddding-top: 10px;',
                        'placeholder'           => '',
                        'label'                 => __('Skal kosten på frakten gå til 0 ved gitt ordre verdi?', 'logistra-robots'),
                        'description'           => __('Inkludert MVA dersom du har valgt avgift på frakt. Eksludert MVA dersom du har ikke-avgift.', 'logistra-robots'),
                        'default'               => 'no',
                        'desc_tip'              => true,
                        'custom_attributes'     => array(),
                        'disabled'              => false,
                    ),
                    'free_value' => array(
                        'title'                 => __('Gratis frakt terskel verdi på ordre', 'logistra-robots'),
                        'type'                  => 'number',
                        'placeholder'           => '',
                        'description'           => "Dersom verdien av ordren blir lik eller over dette tallet vil kostnaden på ordren gå til 0.",
                        'default'               => '0',
                        'desc_tip'              => true
                    ),
                    'max_value_treshhold' => array(
                        'title'                 => __('Skjul for kostbare ordre?', 'logistra-robots'),
                        'type'                  => 'checkbox',
                        'class'                 => '',
                        'css'                   => 'width: 16px; paddding-top: 10px;',
                        'placeholder'           => '',
                        'label'                 => __('Skjuler frakmetoden dersom verdien av ordren overstiger. Angi verdi i neste felt.', 'logistra-robots'),
                        'description'           => __('Noen transportprodukter som f eks "Pakke i postkassen", "Pose på døren",  "Mypack Home Small" eller "helthjem" er ikke beregenet for kostbare forsendelser. Med denne instillingen kan du skjule fraktmetoden for ordre over hvis verdi.', 'logistra-robots'),
                        'default'               => 'no',
                        'desc_tip'              => true,
                        'custom_attributes'     => array(),
                        'disabled'              => false,
                    ),
                    'max_value' => array(
                        'title'                 => __('Kostbar ordre verdi terksel', 'logistra-robots'),
                        'type'                  => 'number',
                        'placeholder'           => '',
                        'description'           => "Dersom verdien av ordren blir over dette tallet vil ikke fraktmetoden bli tilgjengelig for kunde.",
                        'default'               => '0',
                        'desc_tip'              => true
                    ),
                );
            } // End init_form_fields()
        }
    }
}
