<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://www.wildrobot.app/cargonizer
 * @since             1.0.0
 * @package           Logistra_Robots
 *
 * @wordpress-plugin
 * Plugin Name:       Freight Integration for Woocommerce & Logistra Cargonizer by WildRobot
 * Plugin URI:        https://www.wildrobot.app/cargonizer
 * Description:       Integration between WooCommerce and Logistra Cargonizer - Freight administration made easy!
 * Version:           2.3.0
 * Author:            Robertosnap
 * Author URI:        https://www.wildrobot.app
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       logistra-robots
 * Domain Path:       /languages
 * Requires at least: 5.1
 * Tested up to: 5.3.2
 * WC requires at least: 3.0.0
 * WC tested up to: 3.9.3
 */

// If this file is called directly, abort.
if (!defined('WPINC')) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define('LOGISTRA_ROBOTS_PLUGIN_VERSION', '2.3.0');


/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-logistra-robots-activator.php
 */
function activate_logistra_robots()
{
	require_once plugin_dir_path(__FILE__) . 'includes/class-logistra-robots-activator.php';
	Logistra_Robots_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-logistra-robots-deactivator.php
 */
function deactivate_logistra_robots()
{
	require_once plugin_dir_path(__FILE__) . 'includes/class-logistra-robots-deactivator.php';
	Logistra_Robots_Deactivator::deactivate();
}

register_activation_hook(__FILE__, 'activate_logistra_robots');
register_deactivation_hook(__FILE__, 'deactivate_logistra_robots');

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path(__FILE__) . 'includes/class-logistra-robots.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_logistra_robots()
{
	/* Check if woocommerce is active */
	if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
		add_action('admin_notices', function () {
			$class = 'notice notice-error';
			$message = __('Du må aktivere Woocommerce for å bruke Logistra integrasjonen.', 'logistra-robots');

			printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), esc_html($message));
		});
		add_action('admin_init', function () {
			deactivate_plugins(plugin_basename(__FILE__), true);
		});
		return;
	}
	// Maybe show notice adjust settings
	if (get_option('logistra-robots-apikey', null) === null || get_option('logistra-robots-apikey', null) === "") {
		add_action('admin_notices', function () {
			$class = 'notice notice-warning';
			$message = __('Du må konfigurere Transport integrasjonen (Logistra WildRobot). ', 'logistra-robots');
			$url = admin_url("admin.php?page=wc-settings&tab=logistra_robots_tab");
			$link_text = __('Trykk her. ', 'logistra-robots');

			printf('<div class="%1$s"><p>%2$s <a href="%3$s">%4$s</a></p></div>', esc_attr($class), esc_html($message), esc_url($url), esc_html($link_text));
		});
	} else if (get_option('logistra-robots-senderid', null) === null || get_option('logistra-robots-senderid', null) === "") {
		add_action('admin_notices', function () {
			$class = 'notice notice-warning';
			$message = __('Du må velge avsender for dine transporter (Logistra WildRobot).', 'logistra-robots');
			$url = admin_url("admin.php?page=wc-settings&tab=logistra_robots_tab");
			$link_text = __('Trykk her. ', 'logistra-robots');

			printf('<div class="%1$s"><p>%2$s <a href="%3$s">%4$s</a></p></div>', esc_attr($class), esc_html($message), esc_url($url), esc_html($link_text));
		});
	} else if (get_option('logistra-robots-printer-default', null) === null || get_option('logistra-robots-printer-default', null) === "") {
		// add_action('admin_notices', function () {
		// 	$class = 'notice notice-warning';
		// 	$message = __('Du må velge en standard etikette skriver for dine transporter (Logistra WildRobot)', 'logistra-robots');
		// 	$url = admin_url("admin.php?page=wc-settings&tab=logistra_robots_tab");
		// 	$link_text = __('Trykk her. ', 'logistra-robots');

		// 	printf('<div class="%1$s"><p>%2$s <a href="%3$s">%4$s</a></p></div>', esc_attr($class), esc_html($message), esc_url($url), esc_html($link_text));
		// });
	}

	// Update version and maybe migrate user from version 1 to 2
	// var_dump(get_option('logistra-robots-transport-agreements-hash', '1'));
	if (get_option('logistra-robots-current-version') === false) {
		update_option('logistra-robots-current-version', LOGISTRA_ROBOTS_PLUGIN_VERSION);
		$proof_of_version_1 = get_option('logistra-robots-transport-agreements-hash');
		if ($proof_of_version_1 !== false) {
			$token_now_as_username =  get_option('logistra-robots-token');
			if ($token_now_as_username !== false) {
				update_option('logistra-robots-wildrobot-username', $token_now_as_username);
				delete_option('logistra-robots-transport-agreements-hash');
			}
		}
	}

	if (get_option('logistra-robots-current-version') === false) {
		update_option('logistra-robots-current-version', LOGISTRA_ROBOTS_PLUGIN_VERSION);
		$proof_of_version_1 = get_option('logistra-robots-transport-agreements-hash');
		if ($proof_of_version_1 !== false) {
			$token_now_as_username =  get_option('logistra-robots-token');
			if ($token_now_as_username !== false) {
				update_option('logistra-robots-wildrobot-username', $token_now_as_username);
				delete_option('logistra-robots-transport-agreements-hash');
			}
		}
	}

	if ( substr(LOGISTRA_ROBOTS_PLUGIN_VERSION,0,3) === '2.3' && get_option('logistra-robots-dismissed-warning-upgrade-2303', false) === false) {
		update_option('logistra-robots-dismissed-warning-upgrade-2303', true);
		add_action('admin_notices', function () {
			$class = 'notice notice-info is-dismissible';
			$message = __('Versjon 2.3.0 har endret kalkuleringen av gratis frakt grenser. Sjekk dette dersom du har satt opp Logistra Woocommerce vekt og gratis frakt fraktmetoder.', 'logistra-robots');

			printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), esc_html($message));
		});
	}

	$plugin = new Logistra_Robots();
	$plugin->run();
}
run_logistra_robots();
