<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://wildrobot.app/logistra
 * @since      1.0.0
 *
 * @package    Logistra_Robots
 * @subpackage Logistra_Robots/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Logistra_Robots
 * @subpackage Logistra_Robots/public
 * @author     Robertosnap <robertosnap+spam@pm.me>
 */
class Logistra_Robots_Public
{

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct($plugin_name, $version)
	{

		$this->plugin_name = $plugin_name;
		$this->version = $version;

		$this->securityNounce = 'randomTextForLogistraIntegration';

		add_filter('woocommerce_checkout_fields', array($this, 'add_service_partner_picker_field'));
	}

	public function add_service_partner_picker_field($fields)
	{
		$fields['billing']['shipping_service_partner'] = array(
			'label'     => __('Utleveringssted', 'logistra-robots'),
			'placeholder'   => _x('Velg utleveringssted', 'placeholder', 'logistra-robots'),
			'required'  => false,
			'class'     => array('form-row-wide hidden'),
			'clear'     => true,
			'type' => 'text',
		);

		return $fields;
	}


	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles()
	{

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Logistra_Robots_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Logistra_Robots_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		// wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/logistra-robots-public.css', array(), $this->version, 'all');
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
	{

		/* 
            react public.js
        */
		$publicJs = $this->get_hashed_js_dir_url('react', 'public');
		wp_enqueue_script($this->plugin_name . '-logistra-public-js', $publicJs, null, $this->version, true);
		wp_localize_script(
			$this->plugin_name . '-logistra-public-js',
			'logistraRobotsPublic',
			array(
				'wc_ajax_url' => WC()->ajax_url(),
				'security' => wp_create_nonce($this->securityNounce),
				'possiblePostcode' => WC()->countries->get_base_postcode(),
				'possibleCountry' => WC()->countries->get_base_country(),
				'pickupPointCart' => get_option('logistra-robots-pickuppoint-cart', 'yes'),
				'pickupPointCheckout' => get_option('logistra-robots-pickuppoint-checkout', 'yes'),
				'closestServicePartner' => __('Velg nærmeste for meg', 'logistra-robots'),
				'woocommercePageTitle' => site_url(),
			)
		);
	}

	public function get_hashed_js_dir_url($folder, $file)
	{
		if (!empty(glob(plugin_dir_path(__FILE__) . 'partials/' . $folder . '/' . $file . '*.js'))) {
			$full_path = glob(plugin_dir_path(__FILE__) . 'partials/' . $folder . '/' . $file . '*.js')[0];
			$app_pos = strpos($full_path, $file . ".");
			$file_name = substr($full_path, $app_pos);
			return plugin_dir_url(__FILE__) . 'partials/' . $folder . '/' . $file_name;
		} else {
			var_dump("Cannot get WildRobot JS files for => " . $folder . '/' . $file);
		}
	}
}
